#include "LCDPlayback.h"

LCD_Playback::LCD_Playback()
{ 
	m_bFirstSongPlayed = false; 
	m_pLastTrack = NULL;
}

void LCD_Playback::Start()
{
    // Register Callback with Foobar2k
	static_api_ptr_t<play_callback_manager> pcm;
	pcm->register_callback(this,
		flag_on_playback_new_track | 
        flag_on_playback_time | 
        flag_on_playback_stop | 
        flag_on_playback_seek |
        flag_on_playback_edited, 
        false);

    // Create Context for LCD
    memset(&m_LCDContext, 0, sizeof(m_LCDContext));	
    m_LCDContext.appFriendlyName = _T(LOGITECH_APP_NAME);
    m_LCDContext.isAutostartable = FALSE;
    m_LCDContext.isPersistent = FALSE;	
	
    // Register Callback for LCD Softkeys (softkeys currently unused)
    m_LCDContext.onConfigure.configCallback = LCDOnConfigureCB;
    m_LCDContext.onConfigure.configContext = this;	

    // Add Screen from Playback Manager to LCD Output
	m_LCDOutput.Initialize(&m_LCDContext);
	m_PlaybackMgr.Initialize();	
	m_PlaybackMgr.SetExpiration(INFINITE);
    m_LCDOutput.AddScreen(&m_PlaybackMgr);
    m_LCDOutput.LockScreen(&m_PlaybackMgr);

	m_pTimerID = SetTimer(NULL, NULL, 100, TimerProc);
}

void LCD_Playback::SetActiveFormat(int iPreset)
{
	for(int i = 0; i < 3; i++)
	{
		m_ActiveTitleFormats[i] = m_TitleFormats[i+(3*iPreset)];
		if(m_bFirstSongPlayed) UpdateSong(m_pLastTrack);
	}
}

void LCD_Playback::on_playback_new_track(metadb_handle_ptr pTrack)
{
	if(!m_bFirstSongPlayed)
	{
		m_bFirstSongPlayed = true; 
		UpdateSong(pTrack);				
		m_PlaybackMgr.SetTrackPosition(0);
		m_PlaybackMgr.SwitchToSongView();
	}
	else
	{
		UpdateSong(pTrack);
	}
	
}

void LCD_Playback::on_playback_edited(metadb_handle_ptr pTrack)
{
	if(!m_bFirstSongPlayed)
	{
		m_bFirstSongPlayed = true; 
		UpdateSong(pTrack);
		m_PlaybackMgr.SwitchToSongView();
	}
	else
	{
		UpdateSong(pTrack);
	}
}

void LCD_Playback::SetFormatStrings(pfc::string8 strings[12])
{
	for(int i = 0; i < 12; i++)
	{
		m_TitleFormats[i] = strings[i];
	}
	if(m_bFirstSongPlayed) UpdateSong(m_pLastTrack);
}

void LCD_Playback::TimerTick()
{	
	m_LCDOutput.Update(GetTickCount());
	m_LCDOutput.Draw();
}


void LCD_Playback::UpdateSong(metadb_handle_ptr pTrack)
{
	if(m_bFirstSongPlayed && pTrack != NULL)
	{
		pfc::string8 txtLine1;
		pfc::string8 txtLine2;
		pfc::string8 txtLine3;

		// Create Line1 Title Format
		service_ptr_t<titleformat_object> line1TitleFormat;
		static_api_ptr_t<titleformat_compiler>()->compile_safe(line1TitleFormat, m_ActiveTitleFormats[0]);
		pTrack->format_title(NULL, txtLine1, line1TitleFormat, NULL);

		// Create Line2 Title Format
		service_ptr_t<titleformat_object> line2TitleFormat;
		static_api_ptr_t<titleformat_compiler>()->compile_safe(line2TitleFormat, m_ActiveTitleFormats[1]);
		pTrack->format_title(NULL, txtLine2, line2TitleFormat, NULL);

		// Create Line3 Title Format
		service_ptr_t<titleformat_object> line3TitleFormat;
		static_api_ptr_t<titleformat_compiler>()->compile_safe(line3TitleFormat, m_ActiveTitleFormats[2]);
		pTrack->format_title(NULL, txtLine3, line3TitleFormat, NULL);

		// Update Track Length
		m_PlaybackMgr.SetTrackLength(pTrack->get_length());

		// Have Playback Manager Redraw Song and Artist Name
		m_PlaybackMgr.UpdateLines(txtLine1, txtLine2, txtLine3);
		m_pLastTrack = pTrack;
	}
}


void LCD_Playback::UpdateSongTime(double dTime)
{
    // Updates Current Track Position in Playback Manager
	m_PlaybackMgr.SetTrackPosition(dTime);
}


void LCD_Playback::Stop()
{	
	// Kill Timer
	KillTimer(NULL, m_pTimerID);

    // Unregister Foobar Callbacks
	static_api_ptr_t<play_callback_manager> pcm;
	pcm->unregister_callback(this);

    // Shutdown LCD Screen Control (for this plugin)
	m_LCDOutput.Shutdown();
	m_PlaybackMgr.Shutdown();

	// Destroy Last Track Information
	if(m_pLastTrack != NULL) m_pLastTrack.release();
}

DWORD LCD_Playback::LCDOnConfigureCB(int connection, const PVOID pContext)
{
    return ERROR_SUCCESS;
}