//Logitech LCD SDK Components
#include "LCDPlaybackManager.h"

// Application Name To Register with Logitech Drivers
#define LOGITECH_APP_NAME "Foobar2K LCD Display"

// Foobar2K Component Variables
#define COMPONENT_NAME "Logitech G15 LCD Display"
#define COMPONENT_VERSION "0.2a"
#define COMPONENT_DESCRIPTION "Displays song information on a\nLogitech G15 keyboard's LCD display."

// Track Query Format Strings
#define ARTIST_FORMAT "%artist%"
#define TITLE_FORMAT "%title%"

void CALLBACK TimerProc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime );

class LCD_Playback : private play_callback 
{
	private:
		char m_szAppletName[MAX_PATH];
		LCDPlaybackManager m_PlaybackMgr;
		CLCDOutput m_LCDOutput;
		lgLcdConnectContext m_LCDContext;
		pfc::string8 m_TitleFormats[12];
		pfc::string8 m_ActiveTitleFormats[3];
		UINT m_pTimerID;
		metadb_handle_ptr m_pLastTrack;
		bool m_bFirstSongPlayed;
				
		static DWORD CALLBACK LCDOnConfigureCB(int connection, const PVOID pContext);
		void UpdateSong(metadb_handle_ptr pTrack);
		void UpdateSongTime(double dTime);
	public:
        LCD_Playback();
		~LCD_Playback() {};

		void Start();
		void Stop();
		void TimerTick();
		void SetFormatStrings(pfc::string8 strings[12]);
		void SetActiveFormat(int iPreset);

		// Used Callbacks
		virtual void on_playback_new_track(metadb_handle_ptr pTrack);
        virtual void on_playback_edited(metadb_handle_ptr pTrack);
		virtual void on_playback_time(double p_time) {UpdateSongTime(p_time);}
        virtual void on_playback_seek(double p_time) {UpdateSongTime(p_time);}
		virtual void on_playback_stop(play_control::t_stop_reason reason) {UpdateSongTime(0);}

		// Unused Callbacks
		virtual void on_playback_dynamic_info_track(const file_info & p_info) {}
		virtual void on_playback_starting(play_control::t_track_command p_command, bool p_paused) {}		
		virtual void on_playback_pause(bool p_state) {}
		virtual void on_playback_dynamic_info(const file_info & p_info) {}
		virtual void on_volume_change(float p_new_val) {}
};
